from .data import dedupe
from .geometry import (
    calculate_change,
    calculate_distance,
    calculate_slope,
    is_in_circle,
    is_in_range,
    rotate_point,
    sort_clockwise,
    translate_point,
)
__all__ = [
    'data',
    'geometry', 
]

